module Hermes
  module RexClient
    class UpdateContactTask

      def update_contact(rex_entity, client = Hermes::RexClient::Client.new, yopmail = false)
        fields = rex_entity['fields']
        return if !fields['email'] 

        if Rails.env.test? || yopmail
          fields['email'] = fields['email'].split('@')[0] + '@yopmail.com'
        end

        contact = User.find_by(email: fields['email'])

        if !contact
          contact = User.create(user_fields(fields).merge(password: Devise.friendly_token[0,20]))
        else
          contact.update(user_fields(fields))
        end

        contact.group_ids = []

        [
          {fieldName: 'id_etablissement_cristal', classname: Hermes::Entity },
          {fieldName: 'id_bailleur_cristal', classname: Hermes::Lessor },
        ].each do |type|
          if !fields[type[:fieldName]].blank?
            if fields[type[:fieldName]].is_a? String
              add_to_groups(fields[type[:fieldName]], contact, type[:classname])
            end
            if fields[type[:fieldName]].is_a? Array
              fields[type[:fieldName]].each do |id|
                add_to_groups(id, contact, type[:classname])
              end
            end
          end
        end
        
        contact.save!
        contact
      end

      def add_to_groups(id, contact, classname)
        entity = classname.find_by(id_cristal: id)

        if !entity.nil? && !contact.group_ids.include?(entity.group_id)
          contact.group_ids = contact.group_ids << entity.group_id
        end 
        
      end

      def user_fields(fields)
        mobileNumber = fields['tel_pro'].blank? ? '1234567890' : fields['tel_pro']
        role = fields['rolecrelocal'].blank? ? 'rce' : fields['rolecrelocal'].downcase
        {
          username: fields['email'],
          first_name: fields['prenom'],
          last_name: fields['nom'],
          email: fields['email'],
          auth_roles: [fields['rolecrelocal']],
          phone_number: fields['tel_pro'],
          provider: HERMES_CONFIG['identity_provider_key'],
          create_user_data: {
            gender: 'M',
            firstName: fields['prenom'],
            lastName: fields['nom'],
            email: fields['email'],
            login: fields['email'],
            mobileNumber: mobileNumber,
            roles: [role]
          },
        }
      end
    end
  end
end
